#!/usr/bin/python
#coding=utf8
import click
import json
from .wsdl import WSDLService, wsdl_from_url
from .server import start_server

EMPTY_SOAP_RESPONSE = """<?xml version="1.0" encoding="utf-8"?>
<soap12:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:soap12="http://www.w3.org/2003/05/soap-envelope">
  <soap12:Body>
  </soap12:Body>
</soap12:Envelope>
"""


@click.group()
def cli():
    pass


@click.command()
@click.argument('wsdl')
def project(wsdl):
    """
    # 通过WSDL生成对应的配置
    """
    wsdl = wsdl_from_url(wsdl)
    service = WSDLService(wsdl)
    config = service.gen_config()
    with open('./config.json', 'w') as f:
        f.write(json.dumps(config))
    with open('./wsdl.xml', 'w') as f:
        f.write(wsdl)
    for port in service.ports():
        with open(f'./{port}.xml', 'w') as f:
            f.write(EMPTY_SOAP_RESPONSE)
    print("Project Generated!")


@click.command()
@click.argument('port', default="8080")
def serv(port):
    """
    # 启动Mock Server
    """
    with open('./config.json', 'r') as f:
        config = json.loads(f.read())
        start_server(port, config)


def main():
    cli.add_command(project)
    cli.add_command(serv)
    cli()

if __name__ == '__main__':
    main()



#coding=utf8

import requests
from lxml import etree
from io import StringIO, BytesIO


def wsdl_from_url(url):
    """
    """
    if url.startswith('http'):
        return requests.get(url).text
    else:
        with open(url, 'r') as f:
            return f.read()


class Soap(object):
    def __init__(self, body):
        self.documents = etree.fromstring(body)

    def port_name(self):
        """获取Soap请求的接口"""
        for headerOrBody in self.documents:
            if headerOrBody.tag.endswith('Body'):
                for port in headerOrBody:
                    return port.tag.split('}')[-1]
        return ""


class WSDLService(object):
    def __init__(self, wsdl_text):
        self.documents = etree.fromstring(wsdl_text.encode('utf-8'))

    def bindings(self):
        """
        获取入口
        """
        for sub in self.documents:
            if sub.tag.endswith('service'):
                service_root = sub
        if service_root:
            for sub in service_root:
                if not sub.tag.endswith('port'):
                    continue
                if sub.attrib['name'].endswith('Soap'):
                    for addr in sub:
                        url = addr.attrib['location']
                        return "/".join(url.split('/')[3:])
    
    def ports(self):
        """获取port"""
        for sub in self.documents:
            if sub.tag.endswith('portType'):
                if sub.attrib['name'].endswith('Soap'):
                    soap_port = sub
        if soap_port:
            for oper in soap_port:
                yield oper.attrib['name']

    def gen_config(self):
        """生成配置"""
        access_point = self.bindings()
        config = {
            access_point:{

            }
        }
        for port in self.ports():
            config[access_point][port] = f"{port}.xml"
        return config



#coding=utf8

import json
import tornado.ioloop
import tornado.web
import tornado.escape
from .wsdl import Soap

class MainHandler(tornado.web.RequestHandler):
    config = {}

    @classmethod
    def setconfig(cls, config):
        cls.config = config

    def get(self):
        if 'wsdl' in self.request.arguments:
            with open('./wsdl.xml') as f:
                wsdl = f.read()
                self.set_header( 'Content-Type', 'application/xml charset=utf-8' )
                self.finish(wsdl)
        else:
            config = json.dumps(MainHandler.config, indent=4)
            self.set_header( 'Content-Type', 'application/json charset=utf-8' )
            self.finish(config)

    def post(self):
        url = self.request.path[1:]
        if url not in MainHandler.config:
            return self.write(f"url:{url} not match any service")
        soap = Soap(self.request.body)
        port = soap.port_name()
        ports = MainHandler.config[url]
        if port not in ports:
            return self.write(f"invalid method {port}")
        file_path = ports[port]
        with open(file_path, 'r') as f:
            self.set_header( 'Content-Type', 'application/xml charset=utf-8' )
            self.finish(f.read())


def make_app():
    return tornado.web.Application([
        (r"/.*", MainHandler),
    ])


def start_server(port, config):
    app = make_app()
    app.listen(int(port))
    MainHandler.setconfig(config)
    tornado.ioloop.IOLoop.current().start()
